#include "lexer.h"
#include "parser.h"
#include <stdio.h>

int main(int argc, const char** argv)
{
    if (argc < 2) {
        printf("usage: ol <file>\n");
        return 1;
    }
    const char* file_path = argv[1];

    FILE* file = fopen(file_path, "r");
    if (!file) {
        printf("could not open file '%s'\n", file_path);
        return 1;
    }

    ErrorCollector errors;
    error_collector_construct(&errors);

    FileCharReader reader;
    file_char_reader_construct(&reader, file);

    Lexer lexer = lexer_create((CharReader*)&reader, &errors);
    Parser parser = parser_create((CharReader*)&reader, lexer, &errors);

    ExprVec ast = parser_parse(&parser);

    String ast_string;
    string_construct(&ast_string);

    expr_vec_stringify(&ast, &ast_string);

    printf("ast = %s\n", ast_string.data);
    string_destroy(&ast_string);

    expr_vec_destroy(&ast);
    file_char_reader_destroy(&reader);
    error_collector_destroy(&errors);
}
