#ifndef PARSER_H
#define PARSER_H

#include "lexer.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

typedef enum {
    ExprTypeError,
    ExprTypeId,
    ExprTypeInt,
    ExprTypeChar,
    ExprTypeString,
    ExprTypeList,
    ExprTypeS,
} ExprType;

typedef struct Expr Expr;

typedef struct {
    Expr* exprs;
    size_t length;
    size_t capacity;
} ExprVec;

void expr_vec_construct(ExprVec* vec);
void expr_vec_destroy(ExprVec* vec);
void expr_vec_push(ExprVec* vec, Expr expr);
void expr_vec_stringify(const ExprVec* vec, String* acc);
void expr_vec_stringify_tree(const ExprVec* vec, String* acc, int depth);

struct Expr {
    ExprType type;
    Pos pos;
    union {
        String id;
        int64_t int_value;
        char char_value;
        String string;
        ExprVec list;
        ExprVec s;
    };
};

Expr error_expr_construct(Pos pos);
Expr id_expr_construct(Pos pos, String value);
Expr int_expr_construct(Pos pos, int64_t value);
Expr char_expr_construct(Pos pos, char value);
Expr string_expr_construct(Pos pos, String value);
Expr list_expr_construct(Pos pos, ExprVec exprs);
Expr s_expr_construct(Pos pos, ExprVec exprs);
void expr_destroy(Expr* expr);
void expr_stringify(const Expr* expr, String* acc);
void expr_stringify_depth(const Expr* expr, String* acc, int depth);

typedef struct {
    const CharReader* reader;
    Lexer lexer;
    ErrorCollector* errors;
    Token current;
} Parser;

Parser
parser_create(const CharReader* reader, Lexer lexer, ErrorCollector* errors);
ExprVec parser_parse(Parser* parser);

Expr parser_parse_expr(Parser* parser);
Expr parser_parse_id(Parser* parser);
Expr parser_parse_int(Parser* parser);
Expr parser_parse_char(Parser* parser);
Expr parser_parse_string(Parser* parser);
Expr parser_parse_list(Parser* parser);
Expr parser_parse_s(Parser* parser);
void parser_step(Parser* parser);

#endif
